/*
    Copyright (  C ) 2003 Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation;
    version 2 of the License.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

*/

#include "kvolumefader_impl.h"

#include <kdebug.h>
#include <qpainter.h>
#include <qfont.h>
#include <qfontmetrics.h>

KVolumeFader_impl::KVolumeFader_impl( QFrame* w ) : Arts::KFrame_impl( w ? w : new KVolumeFader_Widget( 0 ) ), dB2VolCalc( -36, 6 ), _dir( Arts::BottomToTop ), dbmin_inupdate( false ), dbmax_inupdate( false ), volume_inupdate( false ), direction_inupdate( false )
{
kdDebug()<<"KVolumeFader_impl::KVolumeFader_impl( QFrame* w="<<w<<" )"<<endl;
	_vfwidget = static_cast<KVolumeFader_Widget*>( _qwidget );
	_vfwidget->setImpl( this );
}

/// start: will go away
float KVolumeFader_impl::min() { return 0; }
void KVolumeFader_impl::min( float ) { }
float KVolumeFader_impl::max() { return 0; }
void KVolumeFader_impl::max( float ) { }
/// end: will go away

float KVolumeFader_impl::dbmin() { return dB2VolCalc::dbmin; }
void KVolumeFader_impl::dbmin( float n ) {
	if ( n != dB2VolCalc::dbmin && !dbmin_inupdate ) {
		dbmin_inupdate = true;
		dB2VolCalc::dbmin = n;
		dbmin_changed( dbmin() );
		dbmin_inupdate = false;
	}
}
float KVolumeFader_impl::dbmax() { return dB2VolCalc::dbmax; }
void KVolumeFader_impl::dbmax( float n ) {
	if ( n != dB2VolCalc::dbmax && !dbmax_inupdate ) {
		dbmax_inupdate = true;
		dB2VolCalc::dbmax = n;
		dbmax_changed( dbmax() );
		dbmax_inupdate = false;
	}
}

Arts::Direction KVolumeFader_impl::direction() { return _dir; }
void KVolumeFader_impl::direction( Arts::Direction n ) {
	if ( n != _dir && !direction_inupdate ) {
		direction_inupdate = true;
		_dir = n;
		direction_changed( direction() );
		direction_inupdate = false;
	}
}

long KVolumeFader_impl::resolution() { return 0; }
void KVolumeFader_impl::resolution( long ) { }

float KVolumeFader_impl::volume() { return _volume; }
void KVolumeFader_impl::volume( float n ) {
	if ( n!= _volume ) {
		if ( !volume_inupdate ) {
			volume_inupdate = true;
			_volume = n;
			_vfwidget->setValue( amptondb( _volume ) );
kdDebug() << k_funcinfo << " New volume: " << _volume << " that is " << amptodb( _volume ) <<endl;
			volume_inupdate = false;
		} 
	} //else volume_inupdate = false;
}

float KVolumeFader_impl::dbvolume() { return amptodb( _volume ); }
void KVolumeFader_impl::dbvolume( float n ) { if ( !volume_inupdate ) volume( dbtoamp( n ) ); }

void KVolumeFader_impl::nvolume( float n ) {
	if ( !volume_inupdate ) {
	kdDebug() << k_funcinfo << " n=" << n << " that is " << ndbtoamp( n ) << endl;
		volume( ndbtoamp( n ) );
		volume_inupdate = true;
		volume_changed( _volume );
		dbvolume_changed( dbvolume() );
		volume_inupdate = false;
	}
}

void KVolumeFader_impl::constructor( float min, float max, long res, Arts::Direction dir ) {
	this->min( min ); this->max( max ); direction( dir ); resolution( res );
}

KVolumeFader_Widget::KVolumeFader_Widget( QWidget* p, const char* n ) : QFrame( p,n ), _impl( 0 ), _inupdate( false ), _value( -1 ) {
kdDebug() << k_funcinfo << endl;
	setMinimumSize( 10,10 );
}

void KVolumeFader_Widget::setImpl( KVolumeFader_impl* n ) { _impl = n; }

KVolumeFader_Widget::~KVolumeFader_Widget() {
kdDebug() << k_funcinfo << endl;
}

void KVolumeFader_Widget::setDirection( Arts::Direction ) {}

void KVolumeFader_Widget::setValue( float n ) {
	//kdDebug() << k_funcinfo << n << endl;
	if ( n != _value ) {
		_value = n;
		if ( !_inupdate ) {
			_inupdate = true;
			if ( _impl ) _impl->nvolume( _value );
			repaint();
			_inupdate = false;
		}
	}
}

void KVolumeFader_Widget::drawContents( QPainter* p ){
	float h = contentsRect().height() * ( 1 - _value );
	p->setPen( QPen::NoPen );
	p->setBrush( colorGroup().light() );
	p->drawRect( contentsRect().left(), int( h-1 ), contentsRect().width(), -2 );
	p->setBrush( colorGroup().mid() );
	p->drawRect( contentsRect().left(), int( h+1 ), contentsRect().width(), 2 );
	p->setBrush( colorGroup().dark() );
	p->setPen( colorGroup().dark() );
	p->drawLine( contentsRect().left(), int( h ), contentsRect().right(), int( h ) );
}

void KVolumeFader_Widget::mousePressEvent( QMouseEvent* ){
	//kdDebug() << k_funcinfo << endl;
}

void KVolumeFader_Widget::mouseReleaseEvent( QMouseEvent* qme ){
	setValue( 1 - float( qme->y() ) / contentsRect().height() );
}

void KVolumeFader_Widget::mouseMoveEvent( QMouseEvent* qme ){
	setValue( 1 - float( qme->y() ) / contentsRect().height() );
}

void KVolumeFader_Widget::wheelEvent( QWheelEvent* /*qwe*/ ){
kdDebug() << k_funcinfo << endl;
//	_impl->dbvolume( _impl->dbvolume() + qwe->delta() );
}

REGISTER_IMPLEMENTATION( KVolumeFader_impl );

#include "kvolumefader_impl.moc"
// vim: sw=4 ts=4
