/*

    Copyright (C) 2003 Nikolas Zimmermann <wildfox@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

*/
#include <kurl.h>
#include <qfile.h>

#include <fcntl.h>
#include <assert.h>
#include <unistd.h>
#include <stdlib.h>
#include <iostream.h>

#include <sys/stat.h>

#include "globals.h"
#include "readerwriter.h"

ReaderWriter::ReaderWriter()
{
	m_fd = -1;
 	m_stream = 0;
}

ReaderWriter::~ReaderWriter()
{
	if(m_fd != -1)
		close(m_fd);
}

int ReaderWriter::fd() const
{
	return m_fd;
}

void ReaderWriter::setFD(Mode m, int fd)
{
	m_fd = fd;
	
	if(m == ReaderWriter::WRITE)
		m_stream = fdopen(fd, "wb");
	else
		m_stream = fdopen(fd, "rb");
}

FILE *ReaderWriter::openStream() const
{
	return m_stream;
}

bool ReaderWriter::write16Number(Q_UINT16 number) const
{
	if(GLOBAL_SWAP)
		number = ((number >> 8) | (number << 8)) & 0xffff;
		
	return (fwrite(&number, 2, 1, openStream()) == 1);
}

bool ReaderWriter::write32Number(Q_UINT32 number) const
{
	if(GLOBAL_SWAP)
		number = (number >> 24) | ((number >> 8) & 0xff00) | ((number << 8) & 0xff0000L) | (number << 24);

	return (fwrite(&number, 4, 1, openStream()) == 1);
}

bool ReaderWriter::writeString(const QString &string) const
{
	return (fwrite(string.latin1(), 1, string.length(), openStream()) == string.length());
}

bool ReaderWriter::setup(int fd, Mode m)
{
	m_buf = static_cast<char *>(malloc(GLOBAL_BUFFERSIZE));

	if(!m_buf)
	{
		cerr << "Couldn't allocate resources for raw reading!" << endl;
		return false;
	}

	m_signalSize = 2;
	m_size = GLOBAL_BUFFERSIZE;
	m_count = 0;
	m_pos = 0;
	m_eof = false;

	setFD(m, fd);

	return true;
}

bool ReaderWriter::setup(const KURL &file, Mode m)
{
	if(m == ReaderWriter::WRITE)
		return setup(open(QFile::encodeName(file.path()), O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH), m);
	else
		return setup(open(QFile::encodeName(file.path()), O_RDONLY), m);
}

bool ReaderWriter::eof() const
{
	return m_eof;
}
